package ex.reg;

import bayonet.math.SpecialFunctions;
import blang.core.BlangExtensions;
import blang.core.ConstantSupplier;
import blang.core.DeboxedName;
import blang.core.IntVar;
import blang.core.Model;
import blang.core.ModelBuilder;
import blang.core.ModelComponent;
import blang.core.Param;
import blang.core.RealVar;
import blang.inits.Arg;
import blang.inits.DesignatedConstructor;
import blang.types.SpikedRealVar;
import blang.types.StaticUtils;
import blang.types.internals.IntScalar;
import blang.types.internals.RealScalar;
import ca.ubc.stat.blang.StaticJavaUtils;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.function.Supplier;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;

@SuppressWarnings("all")
public class ChallengerSpiked implements Model {
  public static class Builder implements ModelBuilder {
    private boolean fromCommandLine = false;
    
    @Arg
    public Optional<RealVar> intercept;
    
    public ChallengerSpiked.Builder setIntercept(final RealVar intercept) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.intercept = Optional.of(intercept);
      return this;
    }
    
    @Arg
    public Optional<SpikedRealVar> slope;
    
    public ChallengerSpiked.Builder setSlope(final SpikedRealVar slope) {
      // work around typeRef(..) limitation
      Optional<SpikedRealVar> $generated__dummy = null;
      this.slope = Optional.of(slope);
      return this;
    }
    
    @Arg
    public Optional<IntVar> prediction;
    
    public ChallengerSpiked.Builder setPrediction(final IntVar prediction) {
      // work around typeRef(..) limitation
      Optional<IntVar> $generated__dummy = null;
      this.prediction = Optional.of(prediction);
      return this;
    }
    
    @Arg
    public List<IntVar> incidents;
    
    private boolean incidents_initialized = false;
    
    public ChallengerSpiked.Builder setIncidents(final List<IntVar> incidents) {
      incidents_initialized = true;
      this.incidents = incidents;
      return this;
    }
    
    @Arg
    public List<RealVar> temperatures;
    
    private boolean temperatures_initialized = false;
    
    public ChallengerSpiked.Builder setTemperatures(final List<RealVar> temperatures) {
      temperatures_initialized = true;
      this.temperatures = temperatures;
      return this;
    }
    
    public ChallengerSpiked build() {
      // For each optional type, either get the value, or evaluate the ?: expression
      RealVar intercept;
      if (this.intercept != null && this.intercept.isPresent()) {
        intercept = this.intercept.get();
      } else {
        intercept = $generated__14();
      }
      final RealVar __intercept = intercept;
      SpikedRealVar slope;
      if (this.slope != null && this.slope.isPresent()) {
        slope = this.slope.get();
      } else {
        slope = $generated__15(intercept);
      }
      final SpikedRealVar __slope = slope;
      IntVar prediction;
      if (this.prediction != null && this.prediction.isPresent()) {
        prediction = this.prediction.get();
      } else {
        prediction = $generated__16(intercept, slope);
      }
      final IntVar __prediction = prediction;
      if (!fromCommandLine && !incidents_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing incidents");
      final List<IntVar> __incidents = incidents;
      if (!fromCommandLine && !temperatures_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing temperatures");
      final List<RealVar> __temperatures = temperatures;
      // Build the instance after boxing params
      return new ChallengerSpiked(
        __intercept, 
        __slope, 
        __prediction, 
        __incidents, 
        new ConstantSupplier(__temperatures)
      );
    }
  }
  
  @DesignatedConstructor
  public static ChallengerSpiked.Builder builderFromCommandLine() {
    Builder result = new Builder();
    result.fromCommandLine = true;
    return result;
  }
  
  private final RealVar intercept;
  
  public RealVar getIntercept() {
    return intercept;
  }
  
  private final SpikedRealVar slope;
  
  public SpikedRealVar getSlope() {
    return slope;
  }
  
  private final IntVar prediction;
  
  public IntVar getPrediction() {
    return prediction;
  }
  
  private final List<IntVar> incidents;
  
  public List<IntVar> getIncidents() {
    return incidents;
  }
  
  @Param
  private final Supplier<List<RealVar>> $generated__temperatures;
  
  public List<RealVar> getTemperatures() {
    return $generated__temperatures.get();
  }
  
  /**
   * Utility main method for posterior inference on this model
   */
  public static void main(final String[] arguments) {
    StaticJavaUtils.callRunner(Builder.class, arguments);
  }
  
  /**
   * Auxiliary method generated to translate:
   * intercept
   */
  private static RealVar $generated__0(final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return intercept;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0.0
   */
  private static RealVar $generated__1() {
    return new blang.core.RealConstant(0.0);
  }
  
  public static class $generated__1_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__1();
    }
    
    public String toString() {
      return "0.0";
    }
    
    public $generated__1_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 100.0
   */
  private static RealVar $generated__2() {
    return new blang.core.RealConstant(100.0);
  }
  
  public static class $generated__2_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__2();
    }
    
    public String toString() {
      return "100.0";
    }
    
    public $generated__2_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * slope.continuousPart
   */
  private static RealVar $generated__3(final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return slope.continuousPart;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0.0
   */
  private static RealVar $generated__4() {
    return new blang.core.RealConstant(0.0);
  }
  
  public static class $generated__4_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__4();
    }
    
    public String toString() {
      return "0.0";
    }
    
    public $generated__4_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 100.0
   */
  private static RealVar $generated__5() {
    return new blang.core.RealConstant(100.0);
  }
  
  public static class $generated__5_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__5();
    }
    
    public String toString() {
      return "100.0";
    }
    
    public $generated__5_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * slope.selected
   */
  private static IntVar $generated__6(final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return slope.selected;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0.5
   */
  private static RealVar $generated__7() {
    return new blang.core.RealConstant(0.5);
  }
  
  public static class $generated__7_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__7();
    }
    
    public String toString() {
      return "0.5";
    }
    
    public $generated__7_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0 ..< incidents.size
   */
  private static Iterable<Integer> $generated__8(final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    int _size = incidents.size();
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, _size, true);
    return _doubleDotLessThan;
  }
  
  /**
   * Auxiliary method generated to translate:
   * temperatures.get(i)
   */
  private static RealVar $generated__9(final int i, final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    RealVar _get = temperatures.get(i);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * incidents.get(i)
   */
  private static IntVar $generated__10(final int i, final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    IntVar _get = incidents.get(i);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * logistic(intercept + slope.doubleValue * temperature)
   */
  private static RealVar $generated__11(final RealVar temperature, final RealVar intercept, final SpikedRealVar slope) {
    double _doubleValue = slope.doubleValue();
    double _multiply = (_doubleValue * (temperature).doubleValue());
    double _plus = BlangExtensions.operator_plus(intercept, Double.valueOf(_multiply));
    double _logistic = SpecialFunctions.logistic(_plus);
    return new blang.core.RealConstant(_logistic);
  }
  
  public static class $generated__11_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__11(temperature, intercept, slope);
    }
    
    public String toString() {
      return "logistic(intercept + slope.doubleValue * temperature)";
    }
    
    private final RealVar temperature;
    
    private final RealVar intercept;
    
    private final SpikedRealVar slope;
    
    public $generated__11_class(final RealVar temperature, final RealVar intercept, final SpikedRealVar slope) {
      this.temperature = temperature;
      this.intercept = intercept;
      this.slope = slope;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * prediction
   */
  private static IntVar $generated__12(final RealVar intercept, final SpikedRealVar slope, final IntVar prediction, final List<IntVar> incidents, final List<RealVar> temperatures) {
    return prediction;
  }
  
  /**
   * Auxiliary method generated to translate:
   * logistic(intercept + slope.doubleValue * 31)
   */
  private static RealVar $generated__13(final RealVar intercept, final SpikedRealVar slope) {
    double _doubleValue = slope.doubleValue();
    double _multiply = (_doubleValue * 31);
    double _plus = BlangExtensions.operator_plus(intercept, Double.valueOf(_multiply));
    double _logistic = SpecialFunctions.logistic(_plus);
    return new blang.core.RealConstant(_logistic);
  }
  
  public static class $generated__13_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__13(intercept, slope);
    }
    
    public String toString() {
      return "logistic(intercept + slope.doubleValue * 31)";
    }
    
    private final RealVar intercept;
    
    private final SpikedRealVar slope;
    
    public $generated__13_class(final RealVar intercept, final SpikedRealVar slope) {
      this.intercept = intercept;
      this.slope = slope;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__14() {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * new SpikedRealVar
   */
  private static SpikedRealVar $generated__15(final RealVar intercept) {
    SpikedRealVar _spikedRealVar = new SpikedRealVar();
    return _spikedRealVar;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentInt
   */
  private static IntVar $generated__16(final RealVar intercept, final SpikedRealVar slope) {
    IntScalar _latentInt = StaticUtils.latentInt();
    return _latentInt;
  }
  
  /**
   * Note: the generated code has the following properties used at runtime:
   *   - all arguments are annotated with a BlangVariable annotation
   *   - params additionally have a Param annotation
   *   - the order of the arguments is as follows:
   *     - first, all the random variables in the order they occur in the blang file
   *     - second, all the params in the order they occur in the blang file
   * 
   */
  public ChallengerSpiked(@DeboxedName("intercept") final RealVar intercept, @DeboxedName("slope") final SpikedRealVar slope, @DeboxedName("prediction") final IntVar prediction, @DeboxedName("incidents") final List<IntVar> incidents, @Param @DeboxedName("temperatures") final Supplier<List<RealVar>> $generated__temperatures) {
    this.intercept = intercept;
    this.slope = slope;
    this.prediction = prediction;
    this.incidents = incidents;
    this.$generated__temperatures = $generated__temperatures;
  }
  
  /**
   * A component can be either a distribution, support constraint, or another model  
   * which recursively defines additional components.
   */
  public Collection<ModelComponent> components() {
    ArrayList<ModelComponent> components = new ArrayList();
    
    { // Code generated by: intercept ~ Normal(0.0, 100.0)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Normal(
          $generated__0(intercept, slope, prediction, incidents, $generated__temperatures.get()), 
          new $generated__1_class(), 
          new $generated__2_class()
        )
        );
    }
    { // Code generated by: slope.continuousPart ~ Normal(0.0, 100.0)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Normal(
          $generated__3(intercept, slope, prediction, incidents, $generated__temperatures.get()), 
          new $generated__4_class(), 
          new $generated__5_class()
        )
        );
    }
    { // Code generated by: slope.selected ~ Bernoulli(0.5)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Bernoulli(
          $generated__6(intercept, slope, prediction, incidents, $generated__temperatures.get()), 
          new $generated__7_class()
        )
        );
    }
    for (int i : $generated__8(intercept, slope, prediction, incidents, $generated__temperatures.get())) {
      { // Code generated by: incidents.get(i) | RealVar temperature = temperatures.get(i), intercept, slope ~ Bernoulli(logistic(intercept + slope.doubleValue * temperature))
        // Required initialization:
        RealVar temperature = $generated__9(i, intercept, slope, prediction, incidents, $generated__temperatures.get());
        // Construction and addition of the factor/model:
        components.add(
          new blang.distributions.Bernoulli(
            $generated__10(i, intercept, slope, prediction, incidents, $generated__temperatures.get()), 
            new $generated__11_class(temperature, intercept, slope)
          )
          );
      }
    }
    { // Code generated by: prediction | intercept, slope ~ Bernoulli(logistic(intercept + slope.doubleValue * 31))
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.Bernoulli(
          $generated__12(intercept, slope, prediction, incidents, $generated__temperatures.get()), 
          new $generated__13_class(intercept, slope)
        )
        );
    }
    
    return components;
  }
}
